/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/** acide:fr.bigeon.acide.tool.CLIPrompter.java
 * Created on: Jul 31, 2014 */
package fr.bigeon.gclc.prompt;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/** <p>
 * The {@link CLIPrompter} class is a utility class that provides method to
 * prompt the user.
 *
 * @author Emmanuel BIGEON */
public class CLIPrompter {

    @SuppressWarnings("javadoc")
    private static final String BOOL_CHOICES = "promptbool.choices"; //$NON-NLS-1$
    @SuppressWarnings("javadoc")
    private static final String LIST_DISP_KEY = "promptlist.exit.dispkey"; //$NON-NLS-1$
    @SuppressWarnings("javadoc")
    private static final String PROMPT = "prompt.lineprompt"; //$NON-NLS-1$
    /** @param choices the choices
     * @param cancel the cancel option if it exists
     * @return the number of choices plus one */
    private static <U> int listChoices(List<U> choices, String cancel) {
        int index = 0;
        for (U u : choices) {
            System.out.println((index++) + ") " + u); //$NON-NLS-1$
        }
        if (cancel != null) {
            System.out.println((index++) + ") " + cancel); //$NON-NLS-1$
        }
        return --index;
    }

    /** <p>
     * Prompts the user for an input
     * 
     * @return the user input */
    public static String prompt() {

        // TODO Auto-generated method stub
        return prompt(CLIPrompterMessages.getString(PROMPT) + " "); //$NON-NLS-1$
    }

    /** This method prompts for a user input
     * 
     * @param message the prompting message
     * @return the user entered line */
    public static String prompt(String message) {
        String result = ""; //$NON-NLS-1$
        System.out.print(message + " "); //$NON-NLS-1$
        char c;
        try {
            c = (char) System.in.read();
            while (c != System.lineSeparator().charAt(0)) {
                result += c;
                c = (char) System.in.read();
            }
            while (System.in.available() != 0)
                System.in.read();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    /** @param message the prompting message
     * @return the choice */
    public static boolean promptBoolean(String message) {
        String result = prompt(message +
                               CLIPrompterMessages.getString(BOOL_CHOICES));
        boolean first = true;
        String choices = CLIPrompterMessages
                .getString("promptbool.choices.yes1") + //$NON-NLS-1$
                         ", " + //$NON-NLS-1$
                         CLIPrompterMessages
                                 .getString("promptbool.choices.no1"); //$NON-NLS-1$
        while (!(result.equalsIgnoreCase(CLIPrompterMessages
                .getString("promptbool.choices.yes1")) || //$NON-NLS-1$
                 CLIPrompterMessages
                         .getString("promptbool.choices.no1").equalsIgnoreCase( //$NON-NLS-1$
                                 result) ||
                 CLIPrompterMessages
                         .getString("promptbool.choices.no2").equalsIgnoreCase( //$NON-NLS-1$
                                 result) || CLIPrompterMessages.getString(
                "promptbool.choices.yes2").equalsIgnoreCase(result))) { //$NON-NLS-1$
            if (!first) {

                System.out.println(CLIPrompterMessages.getString(
                        "promptbool.choices.invalid", choices)); //$NON-NLS-1$
                result = prompt(message +
                                CLIPrompterMessages.getString(BOOL_CHOICES));
            }
            first = false;
        }
        return result.equalsIgnoreCase(CLIPrompterMessages
                .getString("promptbool.choices.yes1")) || //$NON-NLS-1$
               result.equalsIgnoreCase(CLIPrompterMessages
                       .getString("promptbool.choices.yes2")); //$NON-NLS-1$
    }

    /** @param <U> the type of choices
     * @param choices the list of choices
     * @param message the prompting message
     * @param cancel the cancel option, or null
     * @return the index of the choice */
    @SuppressWarnings("boxing")
    public static <U> Integer promptChoice(List<U> choices, String message,
                                           String cancel) {
        System.out.println(message);
        int index = listChoices(choices, cancel);
        String result = ""; //$NON-NLS-1$
        boolean keepOn = true;
        int r = -1;
        while (keepOn) {
            result = prompt(CLIPrompterMessages.getString(PROMPT));
            try {
                r = Integer.parseInt(result);
                if (r >= 0 && r <= index)
                    keepOn = false;
                else {
                    System.out.println(CLIPrompterMessages.getString(
                            "promptchoice.outofbounds", 0, index)); //$NON-NLS-1$
                    listChoices(choices, cancel);
                }
    
            } catch (NumberFormatException e) {
                keepOn = true;
                System.out.println(CLIPrompterMessages.getString(
                        "promptchoice.formaterr", 0, index)); //$NON-NLS-1$
                listChoices(choices, cancel);
            }
        }
        if (r == index && cancel != null) return null;
        return r;
    }

    /** @param keys the keys to be printed
     * @param choices the real choices
     * @param message the message
     * @param cancel the cancel option, or null
     * @return the choice */
    @SuppressWarnings("boxing")
    public static <U> U promptChoice(List<String> keys, List<U> choices,
                                     String message, String cancel) {
        Integer index = promptChoice(keys, message, cancel);
        if (index == null) return null;
        return choices.get(index);
    }

    /** @param <U> The choices labels type
     * @param <T> The real choices objects
     * @param choices the list of labels (in order to be displayed)
     * @param choicesMap the map of label to actual objects
     * @param message the prompting message
     * @param cancel the cancel option if it exists (null otherwise)
     * @return the chosen object */
    @SuppressWarnings("boxing")
    public static <U, T> T promptChoice(List<U> choices, Map<U, T> choicesMap,
                                        String message, String cancel) {
        System.out.println(message);
        int index = listChoices(choices, cancel);
        String result = ""; //$NON-NLS-1$
        boolean keepOn = true;
        int r = -1;
        while (keepOn) {
            result = prompt(CLIPrompterMessages.getString(PROMPT));
            try {
                r = Integer.parseInt(result);
                if (r >= 0 && r <= index)
                    keepOn = false;
                else {
                    System.out.println(CLIPrompterMessages.getString(
                            "promptchoice.outofbounds", 0, index)); //$NON-NLS-1$
                    listChoices(choices, cancel);
                }
    
            } catch (NumberFormatException e) {
                keepOn = true;
                System.out.println(CLIPrompterMessages.getString(
                        "promptchoice.formaterr", 0, index)); //$NON-NLS-1$
                listChoices(choices, cancel);
            }
        }
        if (r == index && cancel != null) return null;
        return choicesMap.get(choices.get(r));
    }

    /** @param <U> The choices labels type
     * @param <T> The real choices objects
     * @param choicesMap the map of label to actual objects
     * @param message the prompting message
     * @param cancel the cancel option (or null)
     * @return the chosen object */
    public static <U, T> T promptChoice(Map<U, T> choicesMap, String message,
                                        String cancel) {
        return promptChoice(new ArrayList<>(choicesMap.keySet()), choicesMap,
                message, cancel);
    }

    /** @param message the prompt message
     * @return the integer */
    public static int promptInteger(String message) {
        boolean still = true;
        int r = 0;
        while (still) {
            String result = prompt(message);
            try {
                if (result.isEmpty()) {
                    still = true;
                    continue;
                }
                r = Integer.parseInt(result);
                still = false;
            } catch (Exception e) {
                still = true;
            }
        }
        return r;
    }

    /** This methods prompt the user for a list of elements
     * 
     * @param message the message
     * @return the list of user inputs */
    public static List<String> promptList(String message) {
        return promptList(message,
                CLIPrompterMessages.getString("promptlist.exit.defaultkey")); //$NON-NLS-1$
    }

    /** This methods prompt the user for a list of elements
     * 
     * @param message the message
     * @param ender the ending sequence for the list
     * @return the list of user inputs */
    public static List<String> promptList(String message, String ender) {
        List<String> strings = new ArrayList<>();
        System.out.println(message +
                           CLIPrompterMessages.getString(LIST_DISP_KEY, ender));
        String res = null;
        while (!ender.equals(res)) {
            res = CLIPrompter.prompt(CLIPrompterMessages.getString(PROMPT));
            if (!res.equals(ender)) strings.add(res);
        }
        return strings;
    }

    /** Prompt for a text with several lines.
     * 
     * @param message the prompting message
     * @return the text */
    public static String promptLongText(String message) {
        return promptLongText(message,
                CLIPrompterMessages.getString("promptlongtext.exit.defaultkey")); //$NON-NLS-1$
    }

    /** Prompt for a text with several lines.
     * 
     * @param message the prompting message
     * @param ender the ender character
     * @return the text */
    public static String promptLongText(String message, String ender) {
        System.out.println(message +
                           CLIPrompterMessages.getString(
                                   "promptlongtext.exit.dispkey", ender)); //$NON-NLS-1$
        String res = CLIPrompter.prompt(PROMPT);
        String line = res;
        while (!line.equals(ender)) {
            line = CLIPrompter.prompt(PROMPT);
            if (!line.equals(ender)) res += System.lineSeparator() + line;
        }
        return res.equals(ender) ? "" : res; //$NON-NLS-1$
    }

    /** @param prompt the prompting message
     * @param reprompt the prompting message after empty input
     * @return the non empty input */
    public static String promptNonEmpty(String prompt, String reprompt) {
        String res = prompt(prompt);
        while (res.isEmpty())
            res = prompt(reprompt);
        return res;
    }

    /** Utility class hidden constructor */
    private CLIPrompter() {}
}
